# setup.py placed at root directory
from setuptools import setup

setup(
    name='HousingPriceSP',
    version='1.1.0',
    author='Prakash Singh',
    description='Package to Predict housing Price!',
    long_description='This Package will help to predict housing price using regression model and using differnt parameters',
    packages=['HousingPriceSP'],
    python_requires=">=3.6"

)